-- Author: B. Pankiewicz
-- This code can be used free and without any warranty and only if this comment is not removed
-- RS232 receiver with speed autosense option
-- SIGNALS description:
-- clk_i - 50MHz input clock
-- rst_i - asynchrons reset
-- seg3_i,seg2_i,seg1_i,seg0_i - inputs displayed at -7 seg LED display
-- dots_i - switching dots at segments
-- segs_to_on_i switching the segments
-- led7_an_o - LED display segment selection - LOW level active
-- led7_seg_o - 7-seg LED signals - LOW level active

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity LED_driver is
    Port (  clk_i 			: in   STD_LOGIC;
            rst_i 			: in   STD_LOGIC;
				seg3_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg2_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg1_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg0_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				dots_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				segs_to_on_i 	: in   STD_LOGIC_VECTOR (3 downto 0);
				led7_an_o 		: out  STD_LOGIC_VECTOR (3 downto 0);
				led7_seg_o 		: out  STD_LOGIC_VECTOR (7 downto 0));
end LED_driver;

architecture Behavioral of LED_driver is
	-- function which decodes input into 7-seg LEDs, output LOW active, DP always off. 
	function dec_to_seg(input:std_logic_vector(3 downto 0)) return std_logic_vector is
		variable output:std_logic_vector(6 downto 0);
	begin
		output:="0000000";
		case input is
		   --  -------segments: abcdefg     
			when X"0"=> output:="0000001";
			when X"1"=> output:="1001111";
			when X"2"=> output:="0010010";
			when X"3"=> output:="0000110";
			when X"4"=> output:="1001100";
			when X"5"=> output:="0100100";
			when X"6"=> output:="0100000";
			when X"7"=> output:="0001111";
			when X"8"=> output:="0000000";
			when X"9"=> output:="0000100";
			when X"a"=> output:="0001000";
			when X"b"=> output:="1100000";
			when X"c"=> output:="0110001";
			when X"d"=> output:="1000010";
			when X"e"=> output:="0110000";
			when X"f"=> output:="0111000";
			when others => null;
		end case;
		return (output);
	end function;

begin
	-- 1kHz synchronous to clk_i process mutiplexing seg3_i,seg2_i,seg1_i,seg0_i to LEDs
	process (clk_i, rst_i)
		variable cnt:integer range 0 to 50000000;  -- divide variable
		variable act_segment:integer range 0 to 3; -- indicating which segment is actualy active
	begin
		if rst_i='1' then
			led7_an_o <= (others => '1');
			led7_seg_o <= (others => '1');
			cnt:=0;
		elsif Rising_Edge(clk_i) then
			if cnt=49999 then
				cnt:=0;
				if act_segment=3 then
					act_segment:=0;
				else
					act_segment:=act_segment+1;
				end if;
			else
				cnt:=cnt+1;
			end if;
			case act_segment is
				when 0 => 	led7_seg_o <= dec_to_seg(seg0_i) & (not dots_i(0)); 
								if segs_to_on_i(0)='1' then
									led7_an_o <= "1110";
								else
									led7_an_o <= "1111";
								end if;
				when 1 => 	led7_seg_o <= dec_to_seg(seg1_i) & (not dots_i(1)); 
								if segs_to_on_i(1)='1' then
									led7_an_o <= "1101";
								else
									led7_an_o <= "1111";
								end if;
				when 2 => 	led7_seg_o <= dec_to_seg(seg2_i) & (not dots_i(2));
								if segs_to_on_i(2)='1' then
									led7_an_o <= "1011";
								else
									led7_an_o <= "1111";
								end if;
				when 3 => 	led7_seg_o <= dec_to_seg(seg3_i) & (not dots_i(3)); led7_an_o <= "0111";
								if segs_to_on_i(3)='1' then
									led7_an_o <= "0111";
								else
									led7_an_o <= "1111";
								end if;
				when others => null;
			end case;
		end if;
	end process;

end Behavioral;