-- Author: B. Pankiewicz
-- This code can be used free and without any warranty and only if this comment is not removed
-- Main block connecting RS323 receiver, transmiter and LED display driver
-- Design reades in RS232 input [RXD_i] using speed rate selected on [SW_i(3 downto 0)] inputs and then 
-- resends into [TXD_o] previously adding [SW_i(7 downto 4)], additionaly leds shows receved data
-- display following infos at LED display:
-- __________________________________________________________
-- |    LED3   |    LED2  |       LED1       |     LED0     |
-- | RS232 bps | always 0 | received DATA high and low bits |
-- __________________________________________________________

-- Signals description:
-- RS232 receiver with speed selected through SW_i(3)-SW_i(0) switches
-- SIGNALS description:
-- RXD_i - rs232 input signal
-- clk_i - 50MHz input clock
-- rst_i - asynchrons reset
-- led7_an_o - LED display segment selection - LOW level active
-- led7_seg_o - 7-seg LED signals - LOW level active

-- Speed rate coding:
-- | LED display |  bitrate  |   SW_i  |
-- |      F      | 230400bps | 1 1 1 1 | - not working in LAB EA511 due to MINICOM program failure
-- |      E      | 115200bps | 1 1 1 0 |
-- |      D      |  57600bps | 1 1 0 1 |
-- |      C      |  38400bps | 1 1 0 0 |
-- |      B      |  28800bps | 1 0 1 1 |
-- |      A      |  19200bps | 1 0 1 0 |
-- |      9      |  14400bps | 1 0 0 1 |
-- |      8      |   9600bps | 1 0 0 0 |
-- |      7      |   4800bps | 0 1 1 1 |
-- |      6      |   2400bps | 0 1 1 0 |
-- |      5      |   1200bps | 0 1 0 1 |
-- |      4      |    300bps | 0 1 0 0 |
-- |      3      |  not used | 0 0 1 1 |
-- |      2      |  not used | 0 0 1 0 |
-- |      1      |  not used | 0 0 0 1 |
-- |      0      |  not used | 0 0 0 0 |
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity rs232monitor_v2 is
    Port ( clk_i 			: in  	STD_LOGIC;
           rst_i 			: in  	STD_LOGIC;
           RXD_i 			: in  	STD_LOGIC;
			  TXD_o 			: out  	STD_LOGIC;
			  SW_i  			: in 		STD_LOGIC_VECTOR(7 downto 0);
           led7_an_o 	: out  	STD_LOGIC_VECTOR (3 downto 0);
           led7_seg_o 	: out  	STD_LOGIC_VECTOR (7 downto 0);
			  led_o			: out  	STD_LOGIC_VECTOR (7 downto 0));
end rs232monitor_v2;

architecture Behavioral of rs232monitor_v2 is

component LED_driver is
     Port (  clk_i 			: in   STD_LOGIC;
            rst_i 			: in   STD_LOGIC;
				seg3_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg2_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg1_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				seg0_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				dots_i 			: in   STD_LOGIC_VECTOR (3 downto 0);
				segs_to_on_i 	: in   STD_LOGIC_VECTOR (3 downto 0);
				led7_an_o 		: out  STD_LOGIC_VECTOR (3 downto 0);
				led7_seg_o 		: out  STD_LOGIC_VECTOR (7 downto 0));
end component;

component rx232 is
    Port ( clk_i 				: in  STD_LOGIC;
           rst_i 				: in  STD_LOGIC;
			  RXD_i 				: in  STD_LOGIC;
			  SW_i  				: in  STD_LOGIC_VECTOR(3 downto 0);
           data_o 			: out STD_LOGIC_VECTOR (7 downto 0);
           data_received_o	: out STD_LOGIC;
			  rec_data_read_i	: in  STD_LOGIC);
end component;

component tx232 is
    Port ( clk_i 				: in   STD_LOGIC;
           rst_i 				: in   STD_LOGIC;
           TXD_o 				: out  STD_LOGIC;
           SW_i 				: in   STD_LOGIC_VECTOR (3 downto 0);
           data_i 			: in   STD_LOGIC_VECTOR (7 downto 0);
           send_req_i 		: in   STD_LOGIC;
			  send_req_ack_o	: out  STD_LOGIC);
end component;


signal data :std_logic_vector(7 downto 0);
signal data_to_send :std_logic_vector(7 downto 0);
signal data_received : std_logic;
signal data_read_for_sending : std_logic;


begin

-- received data connected to the leds
led_o<= data;
-- calculation of data to be send
data_to_send <= data+SW_i(7 downto 4);

u1: LED_driver
	port map(  
		clk_i 			=> clk_i,
      rst_i 			=> rst_i,
		seg3_i			=> SW_i(3 downto 0),
		seg2_i			=> "0000",
		seg1_i			=> data(7 downto 4),
		seg0_i			=> data(3 downto 0),
		dots_i			=> "0100",
		segs_to_on_i	=> "1011",
		led7_an_o		=> led7_an_o,
		led7_seg_o		=> led7_seg_o);
	
rx:rx232
	port map(clk_i,rst_i,RXD_i,SW_i(3 downto 0),data,data_received,data_read_for_sending);
tx:tx232
	port map(clk_i,rst_i,TXD_o,SW_i(3 downto 0),data_to_send,data_received,data_read_for_sending);

end Behavioral;

