-- Author: B. Pankiewicz
-- This code can be used free and without any warranty and only if this comment is not removed
-- RS232 transmitter with speed selected through SW_i(3)-SW_i(0) switches
-- SIGNALS description:
-- TXD_i - rs232 output signal
-- clk_i - 50MHz input clock
-- rst_i - asynchrons reset
-- data_i - input data
-- send_req_i - send request input which triggers data transsmision
-- send_req_ack_o - send request ack output which informs that transmmision has been started
-- bitrate coding: [SW_i] according to below table:
-- | LED display |  bitrate  |   SW_i  |
-- |      F      | 230400bps | 1 1 1 1 |
-- |      E      | 115200bps | 1 1 1 0 |
-- |      D      |  57600bps | 1 1 0 1 |
-- |      C      |  38400bps | 1 1 0 0 |
-- |      B      |  28800bps | 1 0 1 1 |
-- |      A      |  19200bps | 1 0 1 0 |
-- |      9      |  14400bps | 1 0 0 1 |
-- |      8      |   9600bps | 1 0 0 0 |
-- |      7      |   4800bps | 0 1 1 1 |
-- |      6      |   2400bps | 0 1 1 0 |
-- |      5      |   1200bps | 0 1 0 1 |
-- |      4      |    300bps | 0 1 0 0 |
-- |      3      |  not used | 0 0 1 1 |
-- |      2      |  not used | 0 0 1 0 |
-- |      1      |  not used | 0 0 0 1 |
-- |      0      |  not used | 0 0 0 0 |

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity tx232 is
    Port ( clk_i 				: in   STD_LOGIC;
           rst_i 				: in   STD_LOGIC;
           TXD_o 				: out  STD_LOGIC;
           SW_i 				: in   STD_LOGIC_VECTOR (3 downto 0);
           data_i 			: in   STD_LOGIC_VECTOR (7 downto 0);
           send_req_i 		: in   STD_LOGIC;
			  send_req_ack_o	: out  STD_LOGIC);
end tx232;

architecture Behavioral of tx232 is

	constant max_div_factor: integer:=166667;
	signal div_factor: integer range 0 to max_div_factor;
	signal enable :std_logic;
	signal main_counter: integer range 0 to 15;
	signal data_int :STD_LOGIC_VECTOR (7 downto 0);

begin
-- process reading SW3-0 and setting correct div_factor needed for speed RXD_i selection, 4 times oversampling
process(clk_i, rst_i)
begin
	if rst_i='1' then
		div_factor <= max_div_factor;
	elsif Rising_Edge(clk_i) then
		case SW_i is
			when "1111" => 	div_factor<=50_000_000/230400; 
			when "1110" => 	div_factor<=50_000_000/115200; 
			when "1101" => 	div_factor<=50_000_000/57600; 
			when "1100" => 	div_factor<=50_000_000/38400;
			when "1011" => 	div_factor<=50_000_000/28800;
			when "1010" => 	div_factor<=50_000_000/19200;
			when "1001" => 	div_factor<=50_000_000/14400;
			when "1000" => 	div_factor<=50_000_000/9600;
			when "0111" => 	div_factor<=50_000_000/4800;
			when "0110" => 	div_factor<=50_000_000/2400;
			when "0101" => 	div_factor<=50_000_000/1200;
			when "0100" => 	div_factor<=50_000_000/300;
			when "0011" => 	div_factor<=50_000_000/300; -- not used
			when "0010" => 	div_factor<=50_000_000/300; -- not used
			when "0001" => 	div_factor<=50_000_000/300; -- not used
			when "0000" => 	div_factor<=4; -- not used
			when others => div_factor<=50_000_000/230400;
		end case;
	end if;
end process;

-- process generating enable with abowe set divide factor
process(clk_i, rst_i)
	variable cnt: integer range 0 to max_div_factor-1;
begin
	if rst_i='1' then
		enable <='0';
		cnt:=0;
	elsif Rising_Edge(clk_i) then
		if cnt = div_factor-1 then
			cnt := 0;
			enable <= '1';
		else
			cnt := cnt+1;
			enable <= '0';
		end if;
	end if;
end process;
	
-- main process generation of TXD_o output,
process(clk_i, rst_i)
begin
	if rst_i='1' then
		data_int <= (others =>'0');
		main_counter <= 0;
		TXD_o <= '1';
		send_req_ack_o <= '0';
	elsif Rising_Edge(clk_i) then
		if enable = '1' then
			if (main_counter=0 and send_req_i='0') then
				main_counter <= 0;
			else
				if main_counter=10 then
					main_counter <= 0;
				else
					main_counter <= main_counter+1;
				end if;
				if main_counter=0 then
					data_int <= data_i;
				end if;
			end if;
			case main_counter is
				when  0 => TXD_o <= '1';			-- waiting for transsmision
				when  1 => TXD_o <= '0';			-- START BIT
							  send_req_ack_o <= '1';
				when  2 => TXD_o <= data_int(0); -- DATA (0)
				when  3 => TXD_o <= data_int(1);
				when  4 => TXD_o <= data_int(2);
				when  5 => TXD_o <= data_int(3);
				when  6 => TXD_o <= data_int(4);
				when  7 => TXD_o <= data_int(5);
				when  8 => TXD_o <= data_int(6);
				when  9 => TXD_o <= data_int(7);	-- DATA (7)
				when 10 => TXD_o <= '1';			-- STOP BIT
							  send_req_ack_o <= '0';
				when 11 => TXD_o <= '1';			-- waiting for transsmision
				when others => null;
			end case;
		end if;
	end if;
end process;


end Behavioral;

